<?php

namespace App\Http\Controllers\Api;

use Carbon\Carbon;
use App\Models\Product;

use Illuminate\Http\Request;
use App\Models\ProductDetailKey;
use App\Models\Command;
use App\Models\Price;
use App\Repositories\PriceRepository;
use App\Repositories\CommandStateRepository;
use App\Repositories\EntrepriseRepository;
use App\Repositories\StockOperationRepository;
use App\Http\Controllers\Api\AppBaseController;
use App\Repositories\CommandPriceRepository;
use App\Services\Interfaces\CommandService;
use App\Jobs\SyncCommandsToPos;
use App\Models\StockOperationProduct;

class HomeController extends AppBaseController {

    protected $priceRepository;
    protected $entrepriseRepository;
    protected $stockOperationRepository;
    protected $commandService;

    public function __construct(
        StockOperationRepository $stockOperationRepository, 
        PriceRepository $priceRepository, 
        EntrepriseRepository $entrepriseRepository,
        CommandService $commandService,
        CommandPriceRepository $commandPriceRepository,
        CommandStateRepository $commandStateRepository
        
    ){
        $this->stockOperationRepository = $stockOperationRepository;
        $this->priceRepository = $priceRepository;
        $this->commandStateRepository = $commandStateRepository;
        $this->entrepriseRepository = $entrepriseRepository;
        $this->commandService = $commandService;
        $this->commandPriceRepository = $commandPriceRepository;
    }
    
    public function adminRedirect(){
        return redirect('/admin');
    }
    
    public function getProductsGuests(Request $request){
        
        \Log::info("start getProductsGuests");

        $limit = $request->limit ?? 6;

        $colors = $sizes = $price = $category = $searchKey = null;

        if($filters = $request->filters)
        {
            $colors = array_key_exists('colors', $filters) ? $filters['colors'] : null;
            $sizes  = array_key_exists('sizes', $filters) ? $filters['sizes'] : null;
            $price = array_key_exists('price', $filters) ? $filters['price'] : null;
            $category = array_key_exists('category', $filters) ? $filters['category'] : null;
            $searchKey = array_key_exists('searchkey', $filters) ? $filters['searchkey'] : null;
            $category = $category == 'soldes' ? null : $category;
        }

        $descriptions = \App\Models\ProductDescription::select('id', 'created_at')->whereNull('deleted_at')->where('label', '!=', 'Livraison')->whereHas('availability', function($query) {
            $query->where('is_available', true);
        });
        
        $searchKey && $descriptions = $descriptions->where('label', 'like', "%$searchKey%");

        $category && $descriptions = $descriptions->whereHas('category', function($query) use($category){
            $query
            ->where('tag', $category)
            ->orWhereHas('parent', function($query) use($category){
                $query->where('tag', $category);
            });
        });
        
        \Log::info("start requesting finding descriptions");

        $descriptions = $descriptions->whereHas('products', function($query) use($price, $colors, $sizes)
        {
            $query->select('id', 'file_path', 'product_description_id')
            ->whereHas('prices', function ($query) use($price){
                $query
                ->where("isHidden", false)
                ->where("isStandard", false)
                ->where(function($query){
                    $query->whereHas('tags', function ($query) {
                        $query->where('tag', '!=' ,'delivery_fees');
                    })
                    ->orWhereDoesntHave('tags');
                });
                $price && $query->whereBetween('promoPrice', $price);
            })

            ->whereHas('stock', function($query) {
                $query->where('quantity', '>', 0);
            });

            $colors && $query->whereHas('details', function($query) use($colors){
                $query->whereIn('value', $colors);
            });

            $sizes && $query->whereHas('details', function($query) use($sizes){
                $query->whereIn('value', $sizes);
            });
        })
        ->groupBy('id')
        ->orderBy('created_at', 'desc')
        ->with('products.prices')
        ->paginate($limit);
        
        
        \Log::info("start looping descriptions");
        $data = [];
        foreach ($descriptions as $description) {
            $product = Product::find(collect($description->products)->where('deleted_at', null)->first()->id);
            $price = $product->prices[0];
            $allColors = $this->getColorsByProductDescription($product->product_description_id);
            
            $item['id'] = $product->id;
            $item['price'] = $price->price;
            $item['price_id'] = $price->id;
            $item['product_description_id'] = $product->product_description_id;
            $item['label'] = $product->description->label;
            $item['promoPrice'] = $price->promoPrice;
            $item['category'] = $product->description->category->tag;
            $item['stock'] = $product->stock->quantity;
            $item['productColors'] = $allColors;
            
            if($colors) $item['color'] = $allColors->whereIn('value', $colors)->first();
            else $item['color'] = $allColors->first();
            
            $item['file_path'] = $item['color'] ? $item['color']->product->file_path ?? $product->description->file_path : $product->description->file_path;

            array_push($data, $item);
        }
        
        \Log::info("end getProductsGuests");
        return response()->json([
            'products' => $data,
            'next_page_url' => $descriptions->nextPageUrl(),
            'page' => $descriptions->currentPage(),
            'last' => $descriptions->lastPage()
        ], 200);
    }
    
    
    public function __getProductsGuests(Request $request){
        
        $limit = $request->limit ?? 6;

        $colors = $sizes = $price = $category = $searchKey = null;

        if($filters = $request->filters)
        {
            $colors = array_key_exists('colors', $filters) ? $filters['colors'] : null;
            $sizes  = array_key_exists('sizes', $filters) ? $filters['sizes'] : null;
            $price = array_key_exists('price', $filters) ? $filters['price'] : null;
            $category = array_key_exists('category', $filters) ? $filters['category'] : null;
            $searchKey = array_key_exists('searchkey', $filters) ? $filters['searchkey'] : null;
            $category = $category == 'soldes' ? null : $category;
        }

        $products = \App\Models\Product::select('id', 'file_path', 'product_description_id', 'created_at')
        ->whereHas('description', function($query) use($searchKey, $category){
            $query
                ->whereNull('deleted_at')
                ->where('label', '!=', 'Livraison')
                ->whereHas('availability', function($query) {
                    $query->where('is_available', true);
                });
                $searchKey && $query->where('label', 'like', "%$searchKey%");
                $category && $query->whereHas('category', function($query) use($category){
                    $query
                    ->where('tag', $category)
                    ->orWhereHas('parent', function($query) use($category){
                        $query->where('tag', $category);
                    });
                });
        })
        ->whereHas('prices', function ($query) use($price){
            $query
            ->where("isHidden", false)
            ->where("isStandard", false)
            ->where(function($query){
                $query->whereHas('tags', function ($query) {
                    $query->where('tag', '!=' ,'delivery_fees');
                })
                ->orWhereDoesntHave('tags');
            });
            $price && $query->whereBetween('promoPrice', $price);
        })
        ->whereHas('stock', function($query) {
            $query->where('quantity', '>', 0);
        });

        $colors && $products = $products->whereHas('details', function($query) use($colors){
            $query->whereIn('value', $colors);
        });

        $sizes && $products = $products->whereHas('details', function($query) use($sizes){
            $query->whereIn('value', $sizes);
        });

        $products = $products
                        ->orderBy('created_at', 'desc')
                        ->groupBy('product_description_id')
                        ->paginate($limit);



        $data = [];
        foreach ($products as $product) {

            $price = $product->prices[0];
            $description = $product->description;
            $category = $product->description->category;
            $stock = $product->stock;

            $allColors = $this->getColorsByProductDescription($product->product_description_id);
            
            $item['id'] = $product->id;
            $item['price'] = $price->price;
            $item['price_id'] = $price->id;
            $item['product_description_id'] = $product->product_description_id;
            $item['label'] = $description->label;
            $item['promoPrice'] = $price->promoPrice;
            $item['category'] = $category->tag;
            $item['stock'] = $stock->quantity;
            $item['productColors'] = $allColors;
            
            if($colors) $item['color'] = $allColors->whereIn('value', $colors)->first();
            else $item['color'] = $allColors->first();
            
            $item['file_path'] = $item['color']->product->file_path ?? $product->description->file_path;

            array_push($data, $item);
        }
        
        return response()->json([
            'products' => $data,
            // 'next_page_url' => $products->nextPageUrl()
        ], 200);
    }
    
    
    public function getDescriptionSizes(Request $request) {
        
        $product_description_id = $request->description_id;
        $color_value = $request->header('color');
        $in_stock = true;
        
        $response = ProductDetailKey::select('value', 'id', 'product_id')
        ->whereHas('product', function ($query) use($product_description_id, $color_value, $in_stock){
            $query
            ->where('product_description_id', $product_description_id)

            ->whereHas('prices', function ($query) {
                $query
                ->where("isHidden", false)
                ->where("isStandard", false)
                ->where(function($query){
                    $query->whereHas('tags', function ($query) {
                        $query->where('tag', '!=' ,'delivery_fees');
                    })
                    ->orWhereDoesntHave('tags');
                });
            });

            $color_value && $query->whereHas('detailKeys', function($query) use($color_value){
                $query->where('value', $color_value);
            });

            $in_stock && $query->whereHas('stock', function($query) {
                $query->where('quantity', '>', 0);
            });
        })
        ->whereHas('detailKey', function ($query) {
            $query->where('tag', 'size');
        })
        ->orderBy('id', 'asc')
        ->groupBy('value')
        ->get()->map(function($el) {
            return [
                "id" => $el->id,
                "product_id" => $el->product_id,
                "value" => $el->value,
                "stock" => $el->product->stock->quantity
            ];
        });
        
        return response()->json([
            'sizes' => $response
        ]);
    }


    public function getAllProductsDetailKeys(Request $request) {
        
        function getDetails($tags) {
            return ProductDetailKey::select('value', 'fr_value as label', 'section')
            ->whereHas('product', function ($query) {
                $query            
                ->whereHas('prices', function ($query) {
                    $query
                    ->where("isHidden", false)
                    ->where("isStandard", false)
                    ->where(function($query){
                        $query->whereHas('tags', function ($query) {
                            $query->where('tag', '!=' ,'delivery_fees');
                        })
                        ->orWhereDoesntHave('tags');
                    });
                });
            })
            ->whereHas('detailKey', function ($query) use($tags){
                $query->whereIn('tag', $tags);
            })
            ->where('value', '!=', 'STANDARD')
            
            ->groupBy('value');
        }

        return response()->json([
            "colorsKeys" => getDetails(['color'])->where('fr_value', 'not like', '#%')->orderBy('fr_value', 'asc')->get()->unique('label'),
            "sizesKeys" => getDetails(['size'])->orderBy('value', 'asc')->where('section', '!=', null)->get()->groupBy('section')
        ], 200);
    }


    public function getProductsCompositions(Request $request) {
        
        $limit = $request->limit ?? 6;
        
        $query = \App\Models\Price::whereNotNull('composition_identifier');
        
        if($limit) $query = $query->limit($limit);
        $prices = $query->latest('created_at')->get();
        
        $data = [];
        foreach ($prices as $price) {
            
            $category = $price->category;
            $stock = collect($price->products->map(function($el) {
                return $el->stock->quantity;
            }))->min();
            
            $item['id'] = $price->id;
            $item['price'] = $price->price;
            $item['label'] = $price->label;
            $item['promoPrice'] = $price->promoPrice;
            $item['category'] = $category ? $category->tag : null;
            $item['file_path'] = $price->file_path;
            $item['stock'] = $stock;
            $item['sizes'] = [];
            $item['productColors'] = [];
            
            array_push($data, $item);
        }
        
        return [
            "products" => $data
        ];
    }


    public function addCompositionProductByPrices($price) {

        $opperation_id = \App\Models\StockOperation::first()->id;
        $prices = \App\Models\Price::whereNotNull('composition_identifier')->get();

        function addInstance($price, $opperation_id) {

            $category_id = null;
            $quantity = collect($price->products->map(function($el) {
                return $el->stock->quantity;
            }))->min();

            $productDescription = \App\Models\ProductDescription::create([
                'label' => $price->label,
                'isReturnable' => 1,
                'description' => $price->description,
                'created_at' => $price->created_at,
                'updated_at' => $price->updated_at,
                'deleted_at' => $price->deleted_at,	
                'category_id' => $category_id,
                'brand_id' => 1,
                'file_path' => $price->file_path
            ]);
            $product = \App\Models\Product::create([
                'created_at' => $price->created_at,
                'updated_at' => $price->updated_at,
                'barcode' => $price->barcode,
                'deleted_at' => $price->deleted_at,	
                'product_description_id' => $productDescription->id,
                'tva' => $price->tva
            ]);
            \App\Models\CombinedProduct::create([
                'quantity' => 1,
                'created_at' => $price->created_at,
                'updated_at' => $price->updated_at,
                'deleted_at' => $price->deleted_at,	
                'price_id' => $price->id,
                'product_id' => $product->id,
            ]);
            \App\Models\ProductAvailability::create([
                'product_description_id' => $productDescription->id,
                'is_available' => 1,
                'created_at' => $price->created_at,
                'updated_at' => $price->updated_at,
            ]);
            \App\Models\StockOperationProduct::create([
                'operationId' => $opperation_id,
                'productId'	=> $product->id,
                'quantity' => $quantity,
                'created_at' => $price->created_at,
                'updated_at' => $price->updated_at,
                'deleted_at' => $price->deleted_at,
            ]);
        }

        foreach($prices as $price) addInstance($price, $opperation_id);
    }
    

    public function getProductsHomeTest(Request $request){
        // dd("reached");
        $tags = ["special"];
        
        $descriptionsQuery = \App\Models\ProductDescription::
        with("products")->select('id', 'created_at')->where('label', '!=', 'Livraison')
        ->whereHas('availability', function($query) {
            $query->where('is_available', true);
        })
        ->whereHas('products', function($query) use($tags) {
            $query->select('id', 'file_path', 'product_description_id')
            ->whereHas('prices', function ($query) use($tags){
                $query
                ->where("isHidden", false)
                ->where("isStandard", false);
                
                $tags && $query->whereHas('tags', function ($query) use($tags){
                    $query->whereIn('tag', $tags);
                });
            })
            ->whereHas('stock', function($query) {
                $query->where('quantity', '>', 0);
            });
        })
        ->orderBy('created_at', 'desc')
        ->limit(4);
        // dd($descriptionsQuery->toSql());
        $descriptions = $descriptionsQuery->get();
        
        dump("test");

        // collect($descriptions[0]->products)->where('deleted_at', null)->first()->id
        $data = [];
        foreach ($descriptions as $description) {
            $product = Product::find(collect($description->products)->where('deleted_at', null)->first()->id);
            dd($product);
            $allColors = $this->getColorsByProductDescription($product->product_description_id);
            
            $price = $product->prices[0];
            $item['id'] = $product->id;
            $item['price'] = $price->price;
            $item['price_id'] = $price->id;
            $item['product_description_id'] = $product->product_description_id;
            $item['label'] = $product->description->label;
            $item['promoPrice'] = $price->promoPrice;
            $item['category'] = $product->description->category->tag;
            $item['color'] = collect($allColors)->first();
            $item['file_path'] = $item['color'] ? $item['color']->product->file_path ?? $product->description->file_path : $product->description->file_path;
            $item['stock'] = $product->stock->quantity;

            $item['productColors'] = $this->getColorsByProductDescription($product->product_description_id);
            array_push($data, $item);
        }

        return response()->json([
            'products' => $data
        ], 200);

    }
    public function getProductsHome(Request $request){
        
        $tags = $request->tags != "news" ? $request->tags : null;
        
        if($tags) $tags = explode(",", $request->tags);
        
        $descriptions = \App\Models\ProductDescription::
        with('products','products.prices')->select('id', 'created_at')->where('label', '!=', 'Livraison')
        ->whereHas('availability', function($query) {
            $query->where('is_available', true);
        })
        ->whereHas('products', function($query) use($tags) {
            $query->select('id', 'file_path', 'product_description_id')
            ->whereHas('prices', function ($query) use($tags){
                $query
                ->where("isHidden", false)
                ->where("isStandard", false);
                
                $tags && $query->whereHas('tags', function ($query) use($tags){
                    $query->whereIn('tag', $tags);
                });
            })
            ->whereHas('stock', function($query) {
                $query->where('quantity', '>', 0);
            });
        })
        ->orderBy('created_at', 'desc')
        ->limit(4)
        ->get();

        $data = [];
        foreach ($descriptions as $description) {
            $product = Product::find(collect($description->products)->where('deleted_at', null)->first()->id);
            $allColors = $this->getColorsByProductDescription($product->product_description_id);
            
            $price = $product->prices[0];
            $item['id'] = $product->id;
            $item['price'] = $price->price;
            $item['price_id'] = $price->id;
            $item['product_description_id'] = $product->product_description_id;
            $item['label'] = $product->description->label;
            $item['promoPrice'] = $price->promoPrice;
            $item['category'] = $product->description->category->tag;
            $item['color'] = collect($allColors)->first();
            $item['file_path'] = $item['color'] ? $item['color']->product->file_path ?? $product->description->file_path : $product->description->file_path;
            $item['stock'] = $product->stock->quantity;

            $item['productColors'] = $allColors;
            array_push($data, $item);
        }

        return response()->json([
            'products' => $data
        ], 200);
    }


    public function getProductById(Request $request){

        $product_id = (int) $request->product_id;
        $only_swichting_color = $request->query('onlySwitchingColor');

        $product = Product::with('prices')->select('id', 'file_path', 'product_description_id')
        ->where('id', $product_id)
        ->first();
        

        if(is_null($product)) return; //throw new \Exception("We couldn't find any product id $product_id");

        $data = [];
        $price = $product->prices[0];

        $data['id'] = $product->id;
        $data['file_path'] = $product->file_path ?? $product->description->file_path;
        $data['stock'] = $product->stock->quantity;
        $data['price_id'] = $price->id;
        $data['price'] = $price->price;
        $data['promoPrice'] = $price->promoPrice;
        $data['tags'] = $price->tags->pluck('tag');

        $color = $data['color'] = $product->detailKeys()->select('value', 'id', 'fr_value as label')->whereHas('detailKey', function ($query) {
            $query->where('tag', 'color');
        })->first();    
        
        if($color){
            $data['sizes'] = $this->getSizesByProductDescription($product->product_description_id, $color->value, $in_stock = true) 
            ->with('product.stock', 'product.prices')->get()->map(function($el){
                return [
                    "id" => $el->id,
                    "product_id" => $el->product->id,
                    "value" => $el->value,
                    "stock" => $el->product->stock->quantity,
                    "price_id" => $el->product->prices[0]->id
                ];
            });
        }

        if(is_null($only_swichting_color)){

            $data['label'] = $product->description->label;
            $data['description'] = $product->description->description;
            $data['productColors'] = $this->getColorsByProductDescription($product->product_description_id);
            $data['productSizes'] = $this->getSizesByProductDescription($product->product_description_id)->groupBy('value')->get();
            
            $products_ids = collect($data['productColors'])->pluck('product_id');
            $data['productImages'] = $this->getImagesByProducts($products_ids);
        }

        return response()->json(['product' => $data], 200);
    }


    public function getSelectedProduct(Request $request){

        $description_id = (int) $request->description_id;
        $color_id = (int) $request->color_id;

        $product = Product::select('id', 'file_path', 'product_description_id')
        ->where('product_description_id', $description_id)

        ->whereHas('details', function($query) use($color_id){
            $query->where('ProductDetailkey.id', $color_id);
        })
        ->first();

        if(is_null($product)) throw new \Exception("We couldn't find any product with description id $description_id and color $color_id");

        $data = [];

        $data['id'] = $product->id;
        $data['file_path'] = $product->file_path ?? $product->description->file_path;
        $data['price_id'] = $product->prices[0]->id;
        
        $data['price'] = $product->prices[0]->price;
        $data['promoPrice'] = $product->prices[0]->promoPrice;
        
        $data['stock'] = $product->stock->quantity;
        $data['color'] = $product->detailKeys()->select('value', 'id', 'fr_value as label')->whereHas('detailKey', function ($query) {
            $query->where('tag', 'color');
        })->first();

        // $data['sizes'] = $this->getSizesByProductDescription($product->product_description_id, $data['color']->value, $in_stock = true)
        // ->with('product.stock')->get();

        return response()->json(['product' => $data], 200);
    }


    public function getColorsByProductDescription($product_description_id){

        return ProductDetailKey::with('product:id,file_path')->select('value', 'id', 'product_id')
        ->whereHas('product', function ($query) use($product_description_id){
            $query
            ->where('product_description_id', $product_description_id)

            ->whereHas('prices', function ($query) {
                $query
                ->where("isHidden", false)
                ->where("isStandard", false)
                ->where(function($query){
                    $query->whereHas('tags', function ($query) {
                        $query->where('tag', '!=' ,'delivery_fees');
                    })
                    ->orWhereDoesntHave('tags');
                });
            })

            ->whereHas('stock', function($query) {
                $query->where('quantity', '>', 0);
            });
        })
        ->whereHas('detailKey', function ($query) {
            $query->where('tag', 'color');
        })
        ->orderBy('id', 'asc')
        ->groupBy('value')
        ->get();
    }


    public function getSizesByProductDescription($product_description_id, $color_value = null, $in_stock = null){

        return ProductDetailKey::select('value', 'id', 'product_id')
        ->whereHas('product', function ($query) use($product_description_id, $color_value, $in_stock){
            $query
            ->where('product_description_id', $product_description_id)

            ->whereHas('prices', function ($query) {
                $query
                ->where("isHidden", false)
                ->where("isStandard", false)
                ->where(function($query){
                    $query->whereHas('tags', function ($query) {
                        $query->where('tag', '!=' ,'delivery_fees');
                    })
                    ->orWhereDoesntHave('tags');
                });
            });

            $color_value && $query->whereHas('detailKeys', function($query) use($color_value){
                $query->where('value', $color_value);
            });

            $in_stock && $query->whereHas('stock', function($query) {
                $query->where('quantity', '>', 0);
            });
        })
        ->whereHas('detailKey', function ($query) {
            $query->where('tag', 'size');
        })
        ->orderBy('id', 'asc')
        ->groupBy('value');
    }

    
    public function getImagesByProducts($products_ids){

        return Product::select('id as product_id', 'file_path')->whereIn('id', $products_ids)
        ->whereNotNull('file_path')
        ->whereHas('stock', function($query) {
            $query->where('quantity', '>', 0);
        })
        ->orderBy('product_id', 'asc')
        ->get();  
    }
    
    
    public function getCommandLines(Request $request) {
        

        $commandId = $request->commandId;
        
        $command = Command::where('id', $commandId)->first();
        $commandLines = [];
        
        foreach($command->prices as $price) {
            
            $product = $price->products[0];
            $tags = (array) $price->tags->pluck('tag')->toArray();
            $variants = [];
            
            foreach($product->detailKeys as $detailKey){
                if($detailKey->fr_value && !in_array($detailKey->fr_value, $variants)) array_push($variants, $detailKey->fr_value);
                elseif(!in_array($detailKey->value, $variants) && !$detailKey->fr_value) array_push($variants, $detailKey->value);
            }
            
            $attributes = join('/', $variants);
            
            // if(in_array("delivery_fees", $tags)) continue;
            
            $commandLines[] = [
                "price_id" => $price->id,
                "img_url" => $product->file_path ? $product->file_path : $product->description->file_path,
                "label" => $product->description->label,
                "prix_unit" => $price->promoPrice,
                "keys" => $attributes,
                "quantity" => $price->pivot->quantity,
            ];
        }
        
        // return $commandLines;
        return [
            'lines' => $commandLines,
            'codePromo' => $command->codePromo
        ];
    }
    
    public function getPrice(request $request) {
        
        $priceId = $request->priceId;
        
        $price = Price::where('id', $priceId)->first();
        
        $product = $price->products[0];
        
        $variants = [];
            
        foreach($product->detailKeys as $detailKey){
            if($detailKey->fr_value && !in_array($detailKey->fr_value, $variants)) array_push($variants, $detailKey->fr_value);
            elseif(!in_array($detailKey->value, $variants) && !$detailKey->fr_value) array_push($variants, $detailKey->value);
        }
        
        $attributes = join('/', $variants);
        
        $response = [
            "price_id" => $price->id,
            "img_url" => $product->file_path ? $product->file_path : $product->description->file_path,
            "label" => $product->description->label,
            "keys" => $attributes,
            "prix_unit" => $price->promoPrice,
        ];
        
        return $response;
    }
    
    public function addCloneCommand(request $request) {
        
        $cmdId = $request->cmdId;
        
        $command = Command::find($cmdId);
        \Log::info("Cloning Command: ".$command->trackingNumber);
        
        $commandLines = $request->commandLines;
        
        if(!$command) return;
        
        $payloadLines = [];
        $local_key = \Str::uuid()->toString();
        $deliveryCompany = null;
        $remise = null;
        $commandState = $this->commandStateRepository->getByTag("pending");
        
        $newCommand = $this->commandService->addCommand([
            'adresseId' => $command->adresseId,
            'stateId' => $commandState->id,
            'paymentModeId' => $command->paymentModeId,
            'clientId' => $command->clientId,
            'comment' => $command->comment. '-'.$command->trackingNumber,
            'country' => $command->country,
            'region' => $command->region,
            'city' => $command->city,
            'trackingNumber' => 'w/' . Carbon::now()->format('Y') . '/',
            'code_postal' => $command->code_postal,
            'totalAmount' => $command->totalAmount,
        ]);
        
        $newTracking = 'w/' . Carbon::now()->format('Y') . '/' . $newCommand->id;
        \Log::info("New Tracking: $newTracking");
        
        $newCommand->reduction = $command->reduction;
        $newCommand->codePromo = $command->codePromo;
        $newCommand->trackingNumber = $newTracking;
        $newCommand->save();
        
        
        $cmdRemise = \App\Models\CmdRemise::where('cmdId', $command->id)->first();
        
        if($cmdRemise) {
            \App\Models\CmdRemise::create([
                'cmdId' => $newCommand->id,
                "remiseId" => $cmdRemise["remiseId"],
                "localKey" => $cmdRemise["localKey"],
                "intitule" => $cmdRemise["intitule"],
                "valeur" => $cmdRemise["valeur"]
            ]);	
        }
        
        $this->cancelCommand($command);
        
        $livraisonPrice = 0;
        
        foreach($commandLines as $line) {
            
            $commandPrice = $this->commandPriceRepository->create([
                "clientId" => $newCommand->clientId,
                "commandId" => $newCommand->id,
                "priceId" => $line["price_id"],
                "quantity" => $line["quantity"]
            ]);
            
            $payloadLines[] = [
                'parent_id' => null,
                'id' => 0,
                'created_at' => Carbon::now(),
                'cmd_remise_id' => null,
                'cmd_client_id' => $local_key,
                'local_key' => \Str::uuid()->toString(),
                'updated_at' => Carbon::now(),
                'quantite' => $line["quantity"],
                'prix_prd_cns_prcpl_id' => $line["price_id"],
                'modified_at' => null
            ];
            
            try{
                $price = Price::find($line["price_id"]);
                $product = $price->products[0];
                $productStock = StockOperationProduct::where('productId', $product->id)->first();
                $productStock->quantity -=  $line["quantity"];
                $productStock->save();
                
                $description_label = $product->description->label;
                if(strtoupper($description_label) == 'LIVRAISON') $livraisonPrice = intval($price->price);
        
            }catch(\Exception $error){
                continue;
            }
        }
        
        $totalCommandAmount = $newCommand->total;
        $commandReduction = $request->reduction ? $request->reduction : $newCommand->reduction;
        
        \Log::info("Livraison price : $livraisonPrice");
        
        $totalPrice = intval($newCommand->total) - $livraisonPrice;
        $reductionAmount = intval(($totalPrice * (intval($commandReduction) / 100)));
        
        $newCommand->totalAmount = intval($newCommand->total) - $reductionAmount;
        $newCommand->reduction = $commandReduction;
        
        $newCommand->save();
        
        SyncCommandsToPos::dispatch($newCommand, $payloadLines, $local_key, $deliveryCompany, false, $remise, $reductionAmount)->onQueue('sync-commands');
        
        return $newCommand->id;
    }
    
    public function cancelCommand($command) {
        
        $command->canceled = true;
        $command->save();
        
        $deliveryCompany = null;
        
        $client = new \GuzzleHttp\Client([
            'base_uri' => env('VUE_SYNC_API_URL','https://client.precision.ma'),
            'defaults' => [
                'exceptions' => false
            ],
            'headers' => ['External-Api-Authorization' => env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F')]
        ]);

        $commandDetails = json_decode($client->post(config('urls.command-details'), [
            'json' => [
                'cmdClientIds' =>[ $command->pos_command_id]
            ]
        ])->getBody());
        
        $data = [
            'client_id' => $commandDetails->CmdClient[0]->client_id,
            'stateTag'  => 'validee'
        ];

        $local_key = $commandDetails->CmdClient[0]->local_key;
       
        $commandLines = $commandDetails->CmdPrdCnsPrcpl;

        foreach ($command->lines as $line) {
            try{
                $product = $line->price->products->first();
                $commandPrice = $this->commandPriceRepository->getCommandPrice($command->id, $line->price->id);
                $productStock = StockOperationProduct::where('productId', $product->id)->first();
                $productStock->quantity +=  $commandPrice->quantity;
                $productStock->save();
            }catch(\Exception $erorr) {
                continue;
            }
        }

        SyncCommandsToPos::dispatch($command, $commandLines, $local_key, $deliveryCompany, $data)->onQueue('sync-commands');
    }

}
