<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;

use App\Http\Controllers\Api\AppBaseController;
use App\Models\DeliveryState;
use App\Models\EtatCmd;
use App\Models\ProductAvailability;
use App\Repositories\PriceRepository;
use App\Repositories\PriceReviewRepository;
use Carbon\Carbon;
use App\Repositories\StockOperationRepository;
use App\Repositories\StockOperationProductRepository;
use App\Repositories\StockOperationTypeRepository;
use Illuminate\Support\Facades\Cache;

class ProductController extends AppBaseController
{

    protected $priceRepository;
    protected $priceReviewRepository;
    protected $stockOperationRepository;
    protected $stockOperationProductRepository;
    protected $stockOperationTypeRepository;

    public function __construct(
        PriceRepository $priceRepository,
        PriceReviewRepository $priceReviewRepository,
        StockOperationRepository $stockOperationRepository,
        StockOperationProductRepository $stockOperationProductRepository,
        StockOperationTypeRepository $stockOperationTypeRepository

    ){
        $this->priceRepository = $priceRepository;
        $this->priceReviewRepository = $priceReviewRepository;
        $this->stockOperationRepository = $stockOperationRepository;
        $this->stockOperationProductRepository = $stockOperationProductRepository;
        $this->stockOperationTypeRepository = $stockOperationTypeRepository;
    }


    
    public function createtesFile($file_id, $directory = "DefaultFiles"){
        
        $sync_url = env('VUE_SYNC_API_URL','https://client.precision.ma');
        $api_key = env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F');

        $client = new \GuzzleHttp\Client([
            'base_uri' => $sync_url,
            'defaults' => [
                'exceptions' => false
            ],
            'headers' => ['External-Api-Authorization' => $api_key]
        ]);
        
        try{
            $url = str_replace("{1}", $file_id, config('urls.get-file'));
            $response = $client->get($url);
            $fileName = (string) \Str::uuid();
            $fileExtension = $response->getHeaders()["File-Extension"][0];
            $path = "/$directory/$fileName.$fileExtension";
            \Storage::put('public'.$path, $response->getBody());
            $data->file_path = $path;
        }catch (\Exception $ex) {
            \Log::info("Exception on getting image file $file_id" . $ex->getMessage());
        }
        
        return $path;
    }
    
    
    public function syncPriceFile() {
        
        $lastSyncedDate = \DB::table('settings')->where('key', 'site.last_sync_date')->first();

        $sync_url = env('VUE_SYNC_API_URL','https://client.precision.ma');
        $api_key = env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F');

        $client = new \GuzzleHttp\Client([
            'base_uri' => $sync_url,
            'defaults' => [
                'exceptions' => false
            ],
            'headers' => ['External-Api-Authorization' => $api_key]
        ]);
        
        $criterias['isWeb'] = true;
        $query = '?withoutFilteringOtherStores=true';
        if($lastSyncedDate){
            $criterias['lastUpdatedAt'] = (new Carbon($lastSyncedDate->value))->toISOString();
            $syncDate = (new Carbon($lastSyncedDate->value))->utc()->subHour()->format('Y-m-d\TH:i:s');
            $query .= '&lastUpdatedAt=' . $syncDate;
        }
        
        try{
            $response = $client->get('/catalogueV2AP/PointOfSell/syncPriceFile'. $query, $criterias);
        }catch(\Exception $error){
            dd($error->getMessage());
        }
        
        $data = json_decode($response->getBody());
        
        foreach($data as $el) {
            $path = $this->createtesFile($el->fileId, 'Price');
            \App\Models\Price::where('id', $el->priceId)->update([
                'file_path' => $path
            ]);
        }

        return $data;
    }
    
    
    
    public function syncImageProducts(Request $request) {
        
        $sync_url = env('VUE_SYNC_API_URL','https://client.precision.ma');
        $api_key = env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F');

        $client = new \GuzzleHttp\Client([
            'base_uri' => $sync_url,
            'defaults' => [
                'exceptions' => false
            ],
            'headers' => ['External-Api-Authorization' => $api_key]
        ]);
        
        $productDescriptions = \DB::table('ProductDescription')->where("file_id", ">", "0")->whereNull("file_path")->get();
        $products = \DB::table('Product')->where("file_id", ">", "0")->whereNull("file_path")->get();
        
        foreach($productDescriptions as $product) {
            try{
                $file_id = $product->file_id;
                $url = str_replace("{1}", $file_id, config('urls.get-file'));
                $response = $client->get($url);
                $fileName = (string) \Str::uuid();
                $fileExtension = $response->getHeaders()["file-extension"][0];
                $path = "/pos-images/$fileName.$fileExtension";
                \Storage::put('public'.$path, $response->getBody());
                \DB::table('ProductDescription')->where("id", $product->id)->update([
                    "file_path" => $path
                ]);
            }catch (\Exception $ex) {
                \Log::info("Exception on getting image file $file_id" . $ex->getMessage());
            }
        }
        foreach($products as $product) {
            try{
                $file_id = $product->file_id;
                $url = str_replace("{1}", $file_id, config('urls.get-file'));
                $response = $client->get($url);
                $fileName = (string) \Str::uuid();
                $fileExtension = $response->getHeaders()["file-extension"][0];
                $path = "/pos-images/$fileName.$fileExtension";
                \Storage::put('public'.$path, $response->getBody());
                \DB::table('Product')->where("id", $product->id)->update([
                    "file_path" => $path
                ]);
            }catch (\Exception $ex) {
                \Log::info("Exception on getting image file $file_id" . $ex->getMessage());
            }
        }
        
        return ["status" => "success"];
    }
    
    public function syncDataPrecision(Request $request){

        $auth = auth()->user();
        $input_password = $request->authPassword;
        
        if(!\Hash::check($input_password , $auth->password)){
            $response = [
                'message'    => 'invalide Password',
                'alert-type' => 'error',
            ];

            return redirect()->back()->with($response);
        }
        
        $now = Carbon::now();
        \Log::debug("$auth->name started syncData at : $now");
        
        $lastSyncedDate = \DB::table('settings')->where('key', 'site.last_sync_date')->first();
        

        $sync_url = env('VUE_SYNC_API_URL','https://client.precision.ma');
        $api_key = env('API_AUTHORIZATION','44DB4B49-60F5-4750-8A5A-A4A2A056509F');
        
        $client = new \GuzzleHttp\Client([
            'base_uri' => $sync_url,
            'defaults' => [
                'exceptions' => false
            ],
            'headers' => ['External-Api-Authorization' => $api_key]
        ]);
        
        
        $criterias['isWeb'] = true;
        $query = '?withoutFilteringOtherStores=true';
        if($lastSyncedDate){
            $criterias['lastUpdatedAt'] = (new Carbon($lastSyncedDate->value))->toISOString();
            $syncDate = (new Carbon($lastSyncedDate->value))->utc()->subHour()->format('Y-m-d\TH:i:s');
            $query .= '&lastUpdatedAt=' . $syncDate.'&forceLUPonPrice=true';
        }
        
        try{
            $response = $client->get('/catalogueV2AP/PointOfSell/sync'. $query, $criterias);
        }catch(\Exception $error){
            dd($error->getMessage());
        }

        $data = json_decode($response->getBody());
        
        \DB::statement('SET FOREIGN_KEY_CHECKS=0;');
        $operationType = $this->stockOperationTypeRepository->getDefault();

        $stockOperation = \DB::table('StockOperation')->first();
        if(!$stockOperation) $stockOperation = $this->stockOperationRepository->create([
            "magasinId" => 1,
            "userId" => 1,
            "typeId" => $operationType->id,
            'planifiedFor' => 0
        ]);
        
        
        foreach ($data as $model => $modelData) {
            
            foreach ($modelData as $data) {
                if($model == "PriceCategory" || $model == "PriceFile") {
                    $id = $data->id;
                    $allData = (array) $data;
                    $catExist = \DB::table('PriceCategory')->where('id', $id)->first();
                    if($catExist) \DB::table('PriceCategory')->where('id', $id)->update($allData);
                    else \DB::table('PriceCategory')->insert($allData);
                    continue;
                }

                $id = $data->id;
                $recordExist = \DB::table($model)->where('id', $id)->first();
                if($model == "ProductCategory") $priceCatExist = \DB::table('PriceCategory')->where('id', $id)->first();

                $data->created_at = Carbon::parse($data->created_at)->format('Y-m-d h:i:s');
                $data->updated_at = Carbon::parse($data->updated_at)->format('Y-m-d h:i:s');
                if($model != "PriceTagPrice" && $data->deleted_at) $data->deleted_at = Carbon::parse($data->deleted_at)->format('Y-m-d h:i:s');

                if (property_exists($data, "file_id") && $data->file_id > 0) {
                    try{
                        $file_id = $data->file_id;
                        $url = str_replace("{1}", $file_id, config('urls.get-file'));
                        $response = $client->get($url);
                        $fileName = (string) \Str::uuid();
                        $fileExtension = $response->getHeaders()["file-extension"][0];
                        $path = "/pos-images/$fileName.$fileExtension";
                        \Storage::put('public'.$path, $response->getBody());
                        $data->file_path = $path;
                    }catch (\Exception $ex) {
                        \Log::info("Exception on getting image file $file_id" . $ex->getMessage());
                    }
                }

                $allData = (array) $data;

                if($recordExist || isset($priceCatExist)) \DB::table($model)->where('id', $id)->update($allData);
                else \DB::table($model)->insert($allData);

                if($model == "ProductCategory"){
                    if(isset($priceCatExist)) \DB::table('PriceCategory')->where('id', $id)->update($allData);
                    else \DB::table('PriceCategory')->insert($allData);
                }

                if($model == 'Product' && !$this->stockOperationProductRepository->getProducts($data->id)){
                    $this->stockOperationProductRepository->create([
                        'productId' => $data->id,
                        'operationId' => $stockOperation->id,
                        'quantity' => 0
                    ]);
                }

                if($model == 'ProductDescription'){
                    $productDescriptionAv = ProductAvailability::where('product_description_id', $data->id)->first();
                    if(!$productDescriptionAv) {
                        ProductAvailability::create([
                            'is_available' => false,
                            'product_description_id' => $data->id
                        ]);
                    }
                }
            }

        }
        \DB::statement('SET FOREIGN_KEY_CHECKS=1;');

        $commandTables = json_decode($client->get(config('urls.commandTables'))->getBody());
        $transportCompanies = $commandTables->TransportCompany;
        $deliveryStates = $commandTables->DeliveryState;
        $commandStates = $commandTables->EtatCmd;
        foreach ($deliveryStates as $state) {
            DeliveryState::updateOrCreate([
                'intitule' => $state->intitule,
                'tag' => $state->tag,
            ], [
                'id' => $state->id,
            ]);
        }
        foreach ($transportCompanies as $company) {
            $recordExist = \DB::table('TransportCompany')->where('id', $company->id)->first();
            if(isset($recordExist)) \DB::table('TransportCompany')->where('id', $company->id)->update([
                    'name' => $company->name,
                    'created_at' => $company->created_at,
                    'updated_at' => $company->updated_at,
                    'deleted_at' => $company->deleted_at
                ]);
            else \DB::table('TransportCompany')->insert([
                'id' => $company->id,
                'name' => $company->name,
                'created_at' => $company->created_at,
                'updated_at' => $company->updated_at,
                'deleted_at' => $company->deleted_at
            ]);
        }
        foreach ($commandStates as $commandState) {
            EtatCmd::updateOrCreate([
                'id' => $commandState->id,
                'intitule' => $commandState->intitule,
                'tag' => $commandState->tag,
                'commentaire' => $commandState->commentaire ?? "---",
                'created_at' => $commandState->createdAt,
                'updated_at' => $commandState->updatedAt,
                'deleted_at' => $commandState->deletedAt
            ], [
                'id' => $commandState->id,
            ]);
        }
        
/*        try{
            $this->syncPriceFile();
        }catch(\Exception $error){
            \Log::info('[ERROR] Failed to sync Price File');
        }*/
        
        
        \DB::table('settings')->where('key', 'site.last_sync_date')->update([
            'value' => Carbon::now()->format('Y-m-d H:i:s'),
        ]);
        
        // syncProductsStock();

        $now = Carbon::now();
        \Log::debug("$auth->name finished syncData at : $now");

        $response = [
            'message'    => 'La synchronisation à été traitée',
            'alert-type' => 'success',
        ];

        Cache::flush();

        return redirect()->back()->with($response);
    }
}
