<?php

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/
// Route::get("/test",function(){
//    return "test"; 
// });

Route::get('products/home/test', 'Api\HomeController@getProductsHomeTest');
Route::group(['prefix' => 'auth'], function() {

    Route::post('login', 'Auth\LoginController@loginWithToken');
    Route::post('forgot', 'Auth\LoginController@forgotPassword');

});

// Route::get('status', function(){
//    return ["status" => "healthy wealthy"];
// });

///home routes and urls
Route::group(['prefix' => 'home'], function() {
    Route::post('products/guests/v2', 'Api\HomeController@getProductsGuests')->middleware('cache.response:720');
    Route::get('products/home', 'Api\HomeController@getProductsHome')->middleware('cache.response:0');
    
    
    Route::get('products/home/{description_id}/{color_id}', 'Api\HomeController@getSelectedProduct');//->middleware('cache.response:1');
    Route::get('products/{product_id}', 'Api\HomeController@getProductById');//->middleware('cache.response:1');
    Route::get('description/size/{description_id}', 'Api\HomeController@getDescriptionSizes');

});

Route::get('delivery/all', 'Api\RootAppController@getDeleveryFees');//->middleware('cache.response:1774');
Route::get('command/lines/{commandId}', 'Api\HomeController@getCommandLines');
Route::post('command/clone', 'Api\HomeController@addCloneCommand');
Route::get('price/{priceId}', 'Api\HomeController@getPrice');


Route::get('category/all/test', 'Api\RootAppController@getAllCategoryTest');


Route::group(['prefix' => 'checkout'], function() {

    Route::get('products/simple/guests', 'Api\CheckoutController@getRecommendedProductsGuests');
    Route::get('promo/{promoCode}/guests', 'Api\CheckoutController@getUnauthentifiedPromoCodeReduction');
    Route::post('client/register', 'Api\CheckoutController@addClient');
    Route::get('promo/{promoCode}', 'Api\CheckoutController@getPromoCodeReduction');
    Route::post('/add-payment-file', 'Api\CheckoutController@addPaymentFile');

    Route::group(['middleware' => 'jwt'], function () {
        Route::get('products/simple', 'Api\CheckoutController@getRecommendedProducts');
        Route::post('invite/collegues', 'Api\CheckoutController@inviteCollegues');
        Route::post('cart/submit', 'Api\CheckoutController@addCartcheckOut');
        Route::post('command/{commandId}/cancel', 'Api\CheckoutController@cancelCommand');
        Route::post('/track-commands', 'Api\CheckoutController@getAllCommands');
    });
    Route::post('/command-details', 'Api\CheckoutController@getCommandDetails');
    Route::get('/shipping-products', 'Api\CheckoutController@getShippingPrices');

});

Route::group(['prefix' => 'delivery', 'middleware' => 'jwt'], function() {
    Route::get('commands', 'Api\DeliveryController@getCommands');
    Route::post('command/{commandId}', 'Api\DeliveryController@trackDeliverMan');
    Route::post('command/{commandId}/delivered', 'Api\DeliveryController@markCommandDelivered');
    Route::get('command/{commandId}/position', 'Api\DeliveryController@getCommandPosition');
});

Route::group(['prefix' => 'profile', 'middleware' => 'jwt'], function() {
    Route::get('commands', 'Api\ProfileController@getCommands');
    Route::get('favorite/products', 'Api\ProfileController@getFavoritProducts');
    Route::get('address', 'Api\ProfileController@getAddresses');
    Route::delete('address/{id}', 'Api\ProfileController@deleteAddress');
    Route::get('command/{id}', 'Api\ProfileController@getCommand');
    Route::patch('user/update', 'Api\ProfileController@updateProfile');
});

Route::get('command/handle/{commandId}', 'Voyager\CommandController@handleCommandV2');

Route::group(['prefix' => 'app'], function() {
    Route::post('contact', 'Api\RootAppController@addContact');
    Route::get('init_data', 'Api\RootAppController@getInitData');
    Route::get('stock_status', 'Api\RootAppController@getStockStatus');
    Route::get('reductions', 'Api\RootAppController@getReductions');
    
    Route::get('sliders', 'Voyager\SliderController@getSliders')->middleware('cache.response:1440');

    // Route::get('slidersKeys', 'Voyager\SliderController@setSliderKeys');

    //cache
    Route::get('category/roots', 'Api\RootAppController@getRootCategories')->middleware('cache.response:1440');
    Route::get('category/all', 'Api\RootAppController@getAllCategory')->middleware('cache.response:1440');
    //cache
    //Route::get('slider', 'Api\RootAppController@getSliderByTag')->middleware('cache.response:1440');
    Route::post('product_stock_status', 'Api\RootAppController@getProductStockStatus');
});

Route::group(['prefix' => 'shop'], function() {
    //cache
    Route::get('filters', 'Api\CollectionController@getFilteres');
    Route::get('products', 'Api\CollectionController@getProducts');
    Route::get('ingredients', 'Api\CollectionController@getIngredients');
});

Route::group(['prefix' => 'product'], function() {
    // Route::get('/sync', 'Api\ProductController@syncData');
    //Route::get('{priceId}', 'Api\ProductController@getProductDetail');
    Route::post('{priceId}/review', 'Api\ProductController@addReview');
    //cache
    Route::post('detailKeys', 'Api\HomeController@getAllProductsDetailKeys')->middleware('cache.response:1440');
    //Route::get('detailKeys/{id}', 'Api\HomeController@getProductsDetailKeys')->middleware('cache.response:1440');
});

Route::group(['prefix' => 'app', 'middleware' => 'jwt'], function() {
    Route::get('user', 'Api\RootAppController@getAuthUser');
    Route::get('delivery/user', 'Api\RootAppController@getDeliveryAuthUser');
    Route::get('command/hash', 'Api\RootAppController@getCommandByHash');
    Route::post('price/{priceId}', 'Api\RootAppController@favoritProduct');
    Route::post('user/delete-account', 'Api\RootAppController@deleteAccount');
});

Route::group(['prefix' => 'blog'], function() {
    Route::get('', 'Api\BlogController@getBlogs');
    Route::get('/{slug}', 'Api\BlogController@getBlogDetail');
    Route::post('/send-comment/{blog_id}', 'Api\BlogController@addComment');
});